//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  max7219.c                                                         *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  11/20/2000                                                        *
//*                                                                              *
//* Description:                                                                 *
//*         This is an embedded realtime LED Display application, that uses the  *
//* MAX7219 IC and a PIC18C452 micro-controller from Microchip Technology to     *
//* drive LED displays.  It employs the Shiftout and Pulsout functions to        *
//* communicate to the MAX7219 IC, and uses reverse, itoa and right_justify      *
//* functions to format data.  Currently it handles up to 2 MAX7219 drivers each *
//* capable of driving up to 5 LED digits.  The specs for the MAX7219 specify    *
//* that a maximum of 8 digits may be driven by it, but I have not been able to  *
//* get large 8 digits working at the same time, maybe because the large LED     *
//* digits are drawing too much current.  The floating point I/O routines        *
//* developed in this application are used for formatting and displaying         *
//* data on LED displays.  They may also be used for both the stdio.c and lcd.c  *    
//* applications to implement the printf %f floating point specifier.            *
//*                                                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large LED      *
//* displays; handling multiple MAX7219 devices; using the USART interface to    * 
//* make a serial LED display controller similar to the Scott Edwards serial LCD.*                                                  
//*                                                                              *
//********************************************************************************
   
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18F8520   	          // Register definitions for PIC18F8520 used by Vex Controller
//#include <p18f8520.h>
//#include "ifi_picdefs.h"  // dfr
//#include "timeutil.h"                                    // Header file for timer utilities
 
//#include "def_printf_lib.h"

#include "max7219.h"                                      // Max7219 function headers
//#include "portutil.h"                                     // Port utilities
//#include "display.h"                                      // Realtime display utilities
#include <delays.h>
#endif

#ifdef PIC18F8720     	// Register definitions for PIC18F8720
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include <i2c.h>												   // I2C library functions 
 
#include "my_types.h"                    // My user types
#include "ifi_aliases.h" 




//********************************************************************************
// Constant Definitions
//********************************************************************************  

//#define TEST 
 
// Declare global constants

#define     MAX7219_IC_0   											// Include MAX7219 IC #0
//#define     MAX7219_IC_1   										// Include MAX7219 IC #1

#define   TRUE    	1													// Input direction
#define   FALSE   	0 												// Output direction
#define   INPUT   	1												// Input direction
#define   OUTPUT   	0 												// Output direction

// Define time delays
#define T_2_Us 	1

//*******************************************************************************
// Modes used for shiftin function (similar to BS2 Basic Stamp shiftin instruction)
//*******************************************************************************

#define MSBPRE   	0                                 			// msb-first/pre-clock
#define LSBPRE   	1                                			// lsb-first/pre-clock
#define MSBPOST   2                               				// msb-first/post-clock
#define LSBPOST   3                               				// lsb-first/post-clock

//
// Note:
//     Use pre-clock for the shiftin mode when bits on the device are latched on the falling
// edge (1-0 level transition).  Use post-clock for the shiftin mode when the device latches the
// data onto the serial data output on the rising edge (0-1 level transition)
// of the clock line.
//
//     For more detailed information on the shiftin and Shiftout 
// instructions See Parallax StampII Note 2:"Using Shiftin & Shiftout ", pp. 75

//*******************************************************************************
// Modes used for Shiftout function (similar to BS2 Basic Stamp Shiftout instruction)
//*******************************************************************************

#define LSBFIRST   	0                              			// lsb-first
#define MSBFIRST     1                         					// msb-first



#define		HIGH		1
#define		LOW		0
#define STRING_SIZE  20   // String size for numeric conversions to ascii

#define 		MAXIMUM_DIGITS  10                           // Maximum number of LED digits to be displayed (up to 5 per MAX7219 device)
//#define     STRING_SIZE  20                              // String size for numeric fields 

// Port C Bits
 
#define SPULSE                   PORTCbits.RC5          // Software timing pulse output (used for benchmarking ISRs)

#define TX                       PORTCbits.RC6           // Used for serial communication to PC Host 
#define RX                       PORTCbits.RC7           // Used for serial communication to PC Host 

#define SPULSEDir                TRISCbits.TRISC5        // Software timing pulse output pin direction (used for benchmarking ISRs)

/*  Vex Controller Digital I/O pis
#define IO1             TRISAbits.TRISA0
#define IO2             TRISAbits.TRISA1
#define IO3             TRISAbits.TRISA2
#define IO4             TRISAbits.TRISA3
#define IO5             TRISAbits.TRISA5
#define IO6             TRISFbits.TRISF0
#define IO7             TRISFbits.TRISF1
#define IO8             TRISFbits.TRISF2
#define IO9             TRISFbits.TRISF3
#define IO10            TRISFbits.TRISF4
#define IO11            TRISFbits.TRISF5
#define IO12            TRISFbits.TRISF6
#define IO13            TRISHbits.TRISH4
#define IO14            TRISHbits.TRISH5
#define IO15            TRISHbits.TRISH6
#define IO16            TRISHbits.TRISH7  
 

// Aliases used to drive the pins when used as OUTPUTS. 
#define rc_dig_out01    LATAbits.LATA0
#define rc_dig_out02    LATAbits.LATA1
#define rc_dig_out03    LATAbits.LATA2
#define rc_dig_out04    LATAbits.LATA3
#define rc_dig_out05    LATAbits.LATA5
#define rc_dig_out06    LATFbits.LATF0
#define rc_dig_out07    LATFbits.LATF1
#define rc_dig_out08    LATFbits.LATF2
#define rc_dig_out09    LATFbits.LATF3
#define rc_dig_out10    LATFbits.LATF4
#define rc_dig_out11    LATFbits.LATF5
#define rc_dig_out12    LATFbits.LATF6
#define rc_dig_out13    LATHbits.LATH4
#define rc_dig_out14    LATHbits.LATH5
#define rc_dig_out15    LATHbits.LATH6
#define rc_dig_out16    LATHbits.LATH7

 // Hardware interface with the  MAX7219 (using PIC18C452 Pin numbering)

#define DataPin1       PORTFbits.RF3      // Bits are shifted out this pin # to the MAX7219.
#define ClockPin1      PORTFbits.RF4      // Data valid on rising edge of this clock (pin RD5).
#define LoadPin1       PORTFbits.RF5      // This control pin tells the 7219 to transfer data to LEDs.  

#define DataPin0       PORTFbits.RF3        // Bits are shifted out this pin # to the MAX7219.
#define ClockPin0      PORTFbits.RF4        // Data valid on rising edge of this clock (pin RD5).
#define LoadPin0       PORTFbits.RF5        // This control pin tells the 7219 to transfer data to LEDs.
*/

 // Hardware interface with the  MAX7219 (using PIC18C452 Pin numbering)

#define DataPin1       rc_dig_out09      // Bits are shifted out this pin # to the MAX7219.
#define ClockPin1      rc_dig_out10      // Data valid on rising edge of this clock (pin RD5).
#define LoadPin1       rc_dig_out11      // This control pin tells the 7219 to transfer data to LEDs.  

#define DataPin0       rc_dig_out09        // Bits are shifted out this pin # to the MAX7219.
#define ClockPin0      rc_dig_out10        // Data valid on rising edge of this clock (pin RD5).
#define LoadPin0       rc_dig_out11        // This control pin tells the 7219 to transfer data to LEDs.  

 
 
/*
	Register addresses for the MAX7219 to control a given attribute of the
	display, for instance , its brightness or the number shown in a particular
	digit, you write the register address followed by the data.  For example
	to set the brightness you'd send 'brite//followed by a number from 0 (off)
	to 15 (100% bright.)
*/

#define		decode   	 9									// Decode register, a 1 turns on BCD decoding
#define		brite		10									// Intensity register
#define		scan		11									// scan-limit register
#define		Switch		12									// on/off register
#define		test		15									// Activates test mode (all digits on, 100% bright)


// Declare MAX7219 variables here

byte TheDigit;                                  // Digit to be displayed

char  anum[STRING_SIZE] = "";                   // Numeric field string
char	blank[]="                    ";           // Field of 20 blanks used to reformat numeric I/O
 
//byte  LookupTable[8] = {scan, 3, brite, 15, decode, 0x1F, Switch, 1};     // Lookup table to hold MAX7219 parameters (brightest=15).
//byte  LookupTable[8] = {scan, 3, brite, 3, decode, 0x1F, Switch, 1};      // Lookup table to hold MAX7219 parameters.
byte  LookupTable[8] = {scan, 5, brite, 15, decode, 0x1F, Switch, 1};     // Lookup table to hold MAX7219 parameters.
//byte  LookupTable[8] = {scan,6, brite, 15, decode, 0x1F, Switch, 1};       // Lookup table to hold MAX7219 parameters.
//byte  LookupTable[8] = {scan, 5, test, 15, decode, 0x1F, Switch, 1};      // Lookup table to hold MAX7219 test mode parameters.

//void            itoa(int Value, char *s);
//char            itoh(int x);
//int             htoi(char c);

//  MAX7219 functions are implemented here
void Pulsout(word  Time);
void InitializeMAX7219(void); 
void Shiftout (byte Mode, word Data, byte NumberBits);


#ifdef MAX7219_TEST

char TestNumber[]   = "1234567890";
char TestNumber0[]  = "0987654321";
char TestNumber1[]  = "3.14159263";
char TestNumber2[]  = "2.71828282";
char TestNumber3[]  = "6.23456E28";
char TestNumber4[]  = "-3.14E75";
char TestNumber5[]  = "-2.7815";
char TestNumber6[]  = "14533";
char TestNumberL[]  = "HELP  HELP";

/*
char TestNumber7[]  = "----------";
char TestNumber8[]  = "         1";
char TestNumber9[]  = "        2 ";
char TestNumberA[]  = "       3  ";
char TestNumberB[]  = "      4   ";
char TestNumberC[]  = "     5    ";
char TestNumberD[]  = "    6     ";
char TestNumberE[]  = "   7      ";
char TestNumberF[]  = "  8       ";
char TestNumberG[]  = " 9        ";
char TestNumberH[]  = "0         ";
char TestNumberI[]  = "    5.3   ";
char TestNumberJ[]  = "  2.718   ";
char TestNumberK[]  = "   -4.63  ";
char TestNumberL[]  = "HELP  HELP";
*/ 



#ifdef MAX7219_TEST

//******************************************************************************
//* main - main function
//******************************************************************************  
void            main(void)
{

    int             i;
 

	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
	InitializeUsart();
	
    printf("\r\n MAX7219 LED Controller \r\n"); 

   // Setup timers used for delays and software
	SetupTimers();                                         
                                                         
   // Initialize the MAX7219 Numeric LED Display
   InitializeMAX7219();
  
   // Display the value of PI	
 	LED_Print(TestNumber1);

   while(1);
	
/*
    printf("Testing LED_Print...\r\n");
   	LED_Print(TestNumber7);
	pause(5000);
  	LED_Print(TestNumber8);
	pause(5000);
  	LED_Print(TestNumber9);
	pause(5000);
  	LED_Print(TestNumberA);
	pause(5000);
  	LED_Print(TestNumberB);
	pause(5000);
  	LED_Print(TestNumberC);
	pause(5000);
  	LED_Print(TestNumberD);
	pause(5000);
  	LED_Print(TestNumberE);
	pause(5000);
  	LED_Print(TestNumberF);
	pause(5000);
  	LED_Print(TestNumberG);
	pause(5000);
  	LED_Print(TestNumberH);
	pause(5000);
  	LED_Print(TestNumberI);
	pause(5000);
  	LED_Print(TestNumberJ);
	pause(5000);
  	LED_Print(TestNumberK);
	pause(5000);
 	LED_Print(TestNumberL);
	pause(5000);
*/

   while(1)
   {
      for (i=0; i<99999; i++)
      {
 	
        // Convert the integer i to an ASCII string
        itoa(i, TestNumber);									
 		LED_Print(TestNumber);

        // Wait 10 milliseconds
   		pause(10);
   	  }
	}     
   
#ifdef TEST 

   printf("Testing Pulsout...\r\n");
   
   while(1)
   {
 		Pulsout(ClockPin0, 1); 
		Pulsout(LoadPin0, 1);                
  		Pulsout(DataPin0,1 );  
  		
 		Pulsout(ClockPin1, 1); 
		Pulsout(LoadPin1, 1);                
  		Pulsout(DataPin1, 1 );  

   }
   
   printf("Testing Shiftout ...\r\n");
   
   while(1)
   {
		Shiftout (  MSBFIRST, 0xAA, 8);
		Shiftout (   MSBFIRST, 0xAA, 8);
   }
    

    // Pause for keystroke

    // Ch = getch();

 
   
 
#endif
    
}

#endif  

#endif

//******************************************************************************
//* InitializeMAX7219 - Initialize the MAX7219 device.
//******************************************************************************  
void   InitializeMAX7219(void)
{
 
	byte MaxData; 
	byte Lsb;
   byte Index;												// Index into setup table
 
  
	// I/O Pins used by the Vex Controller
	IO9 = IO10 = IO11 = IO12 = IO13 = 0;

    //TRISA = 0x0000;
    //TRISF = 0x0000;
    //TRISH = 0x0000;

/*
#ifdef PIC18C452 

#ifdef MAX7219_IC_0
 	ClockPinDir0  = OUTPUT;        	        	// Set Clock Pin direction to output 
 	LoadPinDir0 = OUTPUT;            	      // Set Load Pin  direction to output 
 	DataDir0 = OUTPUT;       			        	// Set Data Pin direction to output
#endif

#ifdef MAX7219_IC_1
 	ClockPinDir1  = OUTPUT;        	       	// Set Clock Pin direction to output 
 	LoadPinDir1 = OUTPUT;            	      // Set Load Pin direction to output 
 	DataDir1 = OUTPUT;       			        	// Set Data pin direction to output 
#endif

#endif
*/

    //printf("trace 0: \r\n");
 
    /* Initialize the MAX7219 by using the Lookup table data structure.  Each
     * register is paired with its setting data.  The table sets the scan limit
     * to 4 (5 digits, numbered 0-4); brightness to 3; BCD decoding to the
     * lower 5 digits (the only ones we are displaying), and switches the
     * display on. The MAX7219 expects data in 16-bit packets, but our lookup
     * table holds a series of 8 bit values.  That's why the loop below is
     * designed to pulse the LoadPin line every other byte transmitted.  */


#ifdef MAX7219_IC_0       

    // MAX7219 Device #0 
    
    for (Index = 0; Index < 8; Index++)
    {       
        MaxData = LookupTable[Index];
   
 		  Shiftout (MSBFIRST, MaxData, 8);      // Send the data to the MAX7219 #0

		  // Look at lowest bit of Index

		  Lsb = Index & 1;
        if (Lsb != 0)                               
        {
            Pulsout ( 1);                             // If it's 1, pulse the Load Pin 0 line           
        }
    }
#endif

#ifdef MAX7219_IC_1

    // MAX7219 Device #1

    for (Index = 0; Index < 8; Index++)
    
    {
        MaxData = LookupTable[Index];

		  Shiftout(   MSBFIRST, MaxData, 8);      // Send the data to the MAX7219 #1

		  // Look at lowest bit of Index

	     Lsb = Index & 1;
        if (Lsb != 0)
        {
            Pulsout( 1);                             // If it's 1, pulse the Load Pin 1 line
        }
    }
#endif
}



 
//******************************************************************************
//* LED_Print - Converts a string to separate BCD digits and sends them to the LED 
//* display.  The MAX7219 won't accept a number like 2742 and display it on 
//* the LEDs. Instead it expects the program to send it individual digits preceded 
//* by their position on the display.  It will also send special floating point  
//* characters such as "+","-" "E" and ".".
//******************************************************************************

void     	LED_Print(char *s)
{

	byte NumberDigits;									// Number of digits in string
	byte Digit;												// Current decimal digit value
	byte AsciiDigit;										// Current ASCII digit value
   byte Index;												// Index into setup table
   byte Position;                               // Position of digit
   byte Decimal_Point;                          // Position of decimal point
   byte i;
   char            s1[STRING_SIZE] = "";			// Temporary workspace string
   char            s2[STRING_SIZE] = "";			// Temporary workspace string
   char            s3[STRING_SIZE] = "";			// Temporary workspace string


   // Save original string

   strcpy(s3, s);

   // Find length of numeric string

   NumberDigits = strlen(s);

   right_justify(MAXIMUM_DIGITS, s);                        // Right Justify and fill with blanks

	// Find location of decimal point
	
	Decimal_Point = 255;
	
   for (i=0; i<MAXIMUM_DIGITS; i++)
	{
	   
	   if (s[i] == '.')
		{
			Decimal_Point = i;
         goto ExitLoop;
		}
	}
 
ExitLoop:

	if (Decimal_Point != 255) 
	{
		// Remove decimal point character from string now that the digit position is known

    	strncpy(s1, s, Decimal_Point);
    	strncpy(s2, s+Decimal_Point+1, (strlen(s)-Decimal_Point));
    	strcpy(s, s1);
    	strcat(s, s2);

		// Adjust decimal point digit location.  The decimal point does not count as a separate digit !

		Decimal_Point = (MAXIMUM_DIGITS - Decimal_Point);
   	//right_justify(MAXIMUM_DIGITS, s);                        // Right Justify and fill with blanks
	}
 
 /*
   printf("****************************\r\n");
   printf("trace 0: NumberDigits = ");
   dec(NumberDigits);
   printf(" Decimal_Point = ");
   dec(Decimal_Point); 
   printf("  s = !");
   string(s);
   printf("! \r\n");
*/ 
 
   
    for (Index = 0; Index < MAXIMUM_DIGITS; Index++)
    {
			if (Index < 5)
 			{

				//Position = Index+1; 
				Position = Index; 
 
         	// Digits 0 to 4 using MAX7219 # 0
         	       	
 				Shiftout (   MSBFIRST, Position, 8);      // Send the Index

         	//AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index];                 // Get ASCII Digit
         	AsciiDigit = s[MAXIMUM_DIGITS - Index];                      // Get ASCII Digit
 
         	// Check for special characters
              
         	Digit = TranslateCharacter(AsciiDigit);

				// Turn On the Decimal Point 

  				if (Index == Decimal_Point)  
  				{
             	//AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index + 1];             // Get previous ASCII Digit
           		//Digit = TranslateCharacter(AsciiDigit);
/*
           	   printf("Trace 1: AsciiDigit = !");
            	putch(AsciiDigit);
            	printf("!  Digit = ");
            	dec(Digit);       
            	printf("\r\n");
*/
  					Digit = Digit | BIT_7;										// Set decimal for current digit
  				} 
 
/*        
         	printf("trace 2: Index = ");
         	dec(Index);
         	printf("  AsciiDigit = !");
         	putch(AsciiDigit);
         	printf("!  Digit = ");
         	dec(Digit);       
         	printf("\r\n");
        
*/
  				Shiftout (   MSBFIRST, Digit, 8);      // Send the data  
        		Pulsout( 1);								   			// Pulse the Load Pin line

			}
        	else
 		  	{
        		// Digits 5 to 9 using MAX7219 # 1
        		
 				Position = Index - 4; 
  
				Shiftout (   MSBFIRST, Position, 8);      // Send the Index divided by 2

            AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index];                     // Get ASCII Digit
            //AsciiDigit = s[MAXIMUM_DIGITS - Index];                     // Get ASCII Digit
 
            // Check for special characters
            
            Digit = TranslateCharacter(AsciiDigit);

				// Turn On the Decimal Point 

  				if (Index == Decimal_Point)  
  				{
             	//AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index + 1];             // Get previous ASCII Digit
           		//Digit = TranslateCharacter(AsciiDigit);
/*                     
            	printf("trace 3:  AsciiDigit = !");
            	putch(AsciiDigit);
            	printf("!  Digit = ");
            	dec(Digit);       
            	printf("\r\n");
*/
  					Digit = Digit | BIT_7;										// Set decimal for current digit
  				} 
           
/*                   
            printf("trace 4: Position = ");
            dec(Position);
            printf("\r\n");
            printf("  AsciiDigit = !");
            putch(AsciiDigit);
            printf("!  Digit = ");
            dec(Digit);       
            printf("\r\n");

*/
     			Shiftout (  MSBFIRST, Digit, 8);      // Send the data  
       		Pulsout( 1);								   			// Pulse the Load Pin line   
    
			}
        
    }

   // Restore original string

   strcpy(s, s3);

}

//******************************************************************************
//* TranslateCharacter - Translates the character to a digit 0..,"E",".", or "-".
//******************************************************************************

byte TranslateCharacter(char c)
{
	
   // Check for special characters         
      
   switch (c)
   {
   	case '-':
  		{
  			TheDigit = 10;															// "-" character
  			break;
  		}
  	
  		case 'E':
  		{
  			TheDigit = 11;															// "E" character
  			break;
  		}
  		
   	case 'H':
  		{
  			TheDigit = 12;															// "H" character
  			break;
  		}
  		
   	case 'L':
  		{
  			TheDigit = 13;															// "L" character
  			break;
  		}
  		
   	case 'P':
  		{
  			TheDigit = 14;															// "P" character
  			break;
  		}
  		
  		case ' ': 
  		{
  			TheDigit = 15;															// Blank " " character
 			break;
  		}
  			
  		default:
  		{
     	 	TheDigit =  c - 48;                                     // Convert Digit from ASCII to Binary
      	break;
  		}
  	}
    			
	return TheDigit;
}

/*******************************************************************************
 * right_justify - function to right justify a string.
 *******************************************************************************/

void            right_justify(int size, char *s)

{
    int             len;
    char            s1[STRING_SIZE] = "";
    int             nchars;

  
    len = strlen(s);
    if (len < size)
    {
        nchars = size - len;
        strncpy(s1, blank, nchars);
        strcat(s1, s);
        strcpy(s, s1);
    }
}

 
/*
//*******************************************************************************
//* htoi - function to convert an ascii hex digit to decimal.
//*******************************************************************************

int             htoi(char c)
{
    int             x;

    // Convert invalid characters and blanks to blank code 15 for the display
 
    if ((c >= '0') && (c <= '9'))
      x = (int) c - 48;
    else
      x = 15;

    return (x);
}

//*******************************************************************************
//* itoh - function to convert a decimal to an ascii hex digit.
//*******************************************************************************

char            itoh(int x)
{
    char            c;

    c = (char) (x + 48);
    return (c);
}

 
*/
 
//****************************************************************************************
//* Converts a byte to an ascii string of 2 hex characters and writes                                                          
//* them to the USART.
//****************************************************************************************

int 		hex2(byte Value)
{   
    	HexByte_T HexByte;
 
 	// Convert binary value to an 8-Bit unsigned hex byte

        //btoh(Value, HexByte);  // dfr
 
        //string((char *) HexByte);
 

	return TRUE;
}

                  
 void Pulsout(word  Time)
{	 
	LoadPin0 = 1;
	Delay10TCYx(T_2_Us);
 
	LoadPin0 = 0;
	Delay10TCYx(T_2_Us);
 
}

//******************************************************************************
//* Shiftout - PIC version of Shiftout function that shift bits out synchronously.  
//****************************************************************************** 
void     Shiftout (byte Mode, word Data, byte NumberBits)  
{
   	byte	i;													// Loop index
   	byte DataPortID;											// Data port ID
	byte DataPinID;												// Data pin ID
   	byte ClockPortID;											// Clock port ID
	byte ClockPinID;											// Clock pin ID
    word LastBit;           									// Last bit position (power of 2) 


	// Force Data pin an output

    //DataPinDir0 = 0;
    //DataPinDir1 = 0;

	// Force Clock pin an output

    //ClockPinDir0 = 0;  
    //ClockPinDir1 = 0;  
                                
	// Convert Data pin number to power of 2 for bit position.

	//DataPinID = 1 << DataPin;	

	// Convert Clock pin number to power of 2 for bit position.

//	ClockPinID = 1 << ClockPin;	

	if (Mode == MSBFIRST)
	{
 		// Send data bits to the target using MSB..LSB order.  LastBit represents the most significant bit (MSB).  

 		LastBit = ((word) 1 << (NumberBits - 1));

		for (i = 0; i < NumberBits; i++)
		{
/*
			// Copy parameters to local variables so that they may be examined by MPSIM

   		LocalDataPin = DataPin;
   		LocalClockPin = ClockPin;
   		LocalMode = Mode;
   		LocalData = Data;
   		LocalNumberBits = NumberBits;
*/

			if ((Data & LastBit) == LastBit)
			{
				// Send a logic-1 to target device

				//high(DataPin);
 				DataPin0 = 1; 

			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
 				DataPin0 = 0; 			
			}
   			
   			// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
   
   		
			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			ClockPin0 = 1; 
 
   			// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
  	 
 
			// Clear the clock line

      		//low(ClockPin);
			ClockPin0 = 0; 
 			
  	  	   // Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
   	 
 
			// Get next bit by shifting Data word to the left one bit

			Data = Data << 1;                            
		}
   } 
	else
	{

		// Send data bits to the target using LSB..MSB order  

		for (i = 0; i < NumberBits; i++)
		{
			if (Data & BIT_0 == 1)
			{
				// Send a logic-1 to target device

				//high(DataPin);
				DataPin0 = 1; 

			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
				DataPin0 = 0; 			
			}
			
   			// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
    
 
			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			ClockPin0 = 1; 
			
   			// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
   	 
 
			// Clear the clock line

      		//low(ClockPin);
			ClockPin0  = 0; 			
  
  	   		// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   			Delay10TCYx(T_2_Us);
   		 
 
			// Get next bit by shifting Data word to the right one bit

			Data = Data >> 1;                            
		}
	}
 
}



 
